## Copyright (c) 2012 Miguel Bazdresch
##
## Permission is hereby granted, free of charge, to any person obtaining a
## copy of this software and associated documentation files (the "Software"),
## to deal in the Software without restriction, including without limitation
## the rights to use, copy, modify, merge, publish, distribute, sublicense,
## and/or sell copies of the Software, and to permit persons to whom the
## Software is furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
## FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
## DEALINGS IN THE SOFTWARE.

# Fourier demo with gtk-server
# Note: requires Gaston to be loaded

#push(LOAD_PATH, "/path/to/gaston/jl")
#load("gaston.jl")
load("gtk.jl")

# time plot
function dotimeplot(t,amp,fre,pha)
	plot(1,t,amp*sin(2pi*fre*t+pha),"title","Sinusoidal in time domain","xlabel","Time (s)","ylabel","Amplitude")
end

# plot the spectra
function dofreqplot(f,amp,fre)
	c = zeros(23)
	c[12+fre] = amp/2
	c[12-fre] = amp/2
	plot(2,f,c,"title","Sinusoidal in frequency domain -- magnitude","xlabel","Frequency (Hz)","ylabel","Magnitude","plotstyle","impulses")
end

function dophaseplot(f,fre,pha)
	p = zeros(23)
	p[12+fre] = -pi/2+pha
	p[12-fre] = pi/2-pha
	plot(3,f,p,"title","Sinusoidal in frequency domain -- phase","xlabel","Frequency (Hz)","ylabel","Phase (rad)","plotstyle","impulses")
end

# Main program
# This program interactively plots a sine function in time, and its
# magnitude and phase spectra. The amplitude, frequency and phase of
# sine function are selected through three sliders.
# Please note that the maximum frequency is 10Hz and it may be changed
# in steps of 1Hz; however, for the purpose of finding the Fourier series,
# the fundamental frequency has been set to 1Hz. This is mathematically
# valid but is not the "textbook" way of finding the series. It works
# great for illustrating the time-frequency duality, though.

fifo=gtk_init()
# 'win' is a handle (ID number) to the main GUI window.
gtk_send(fifo,"gtk_init NULL NULL")
win = gtk_send(fifo,"gtk_window_new 0")
# Set window's title, size and position.
# Note that the message sent to gtk-server is always a string.
gtk_send(fifo,"gtk_window_set_title $win 'Fourier Series Demo'")
gtk_send(fifo,"gtk_window_set_default_size $win 300 200")
gtk_send(fifo,"gtk_window_set_position $win 1")
# We'll set our widgets in a table, one widget per row-column
# combination.
tbl = gtk_send(fifo,"gtk_table_new 8 10 0")
gtk_send(fifo,"gtk_container_add $win $tbl")
# Create three labels...
lblA = gtk_send(fifo,"gtk_label_new Amplitude:")
lblF = gtk_send(fifo,"gtk_label_new Frequency:")
lblP = gtk_send(fifo,"gtk_label_new Phase:")
# and put them in specific position on our table.
gtk_send(fifo,"gtk_table_attach_defaults $tbl $lblA 1 3 1 2")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $lblF 1 3 3 4")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $lblP 1 3 5 6")
# horizontal separator
hsep = gtk_send(fifo,"gtk_hseparator_new")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $hsep 1 9 6 7")
# Some Octave advertisement
adv = gtk_send(fifo,"gtk_label_new 'Created with Julia!'")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $adv 1 6 7 8")
# Create (and put on the table) the quit button.
but = gtk_send(fifo,"gtk_button_new_from_stock 'gtk-quit'")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $but 7 9 7 8")
# Create three sliders...
sliA = gtk_send(fifo,"gtk_hscale_new_with_range 1 5 1")
sliF = gtk_send(fifo,"gtk_hscale_new_with_range 1 10 1")
sliP = gtk_send(fifo,"gtk_hscale_new_with_range 0 3.1 0.1")
# and put them on the table.
gtk_send(fifo,"gtk_table_attach_defaults $tbl $sliA 4 9 1 2")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $sliF 4 9 3 4")
gtk_send(fifo,"gtk_table_attach_defaults $tbl $sliP 4 9 5 6")
# Now we need to actually display all the widgets we've created.
gtk_send(fifo,"gtk_widget_show_all $win")

# Set initial values for amplitude, frequency and phase, and display
# the sine function.
f = -11:11
t = 0:0.005:1
amp = 1
fre = 1
pha = 0
dotimeplot(t,amp,fre,pha)
dofreqplot(f,amp,fre)
dophaseplot(f,fre,pha)

# endless loop; we only break out of it when the quit button is clicked
event = ""
while true
    # Up to this point, we have sent commands to gtk-server in order
	# to set up the GUI and display it. However, once the GUI is
	# created, gtk-server will send messages (events) to us whenever
	# the user clicks something on the GUI. In order to read these
	# events, we send gtk-server the message "gtk_server_callback wait".
	# So, if the user has clicked on a widget on the GUI, then
	# 'event' contains the widget's handle (ID number).
	event = gtk_send(fifo,"gtk_server_callback wait")
	# The user clicked on the windows 'close' button.
	if event == win
		break
	end
	# The user clicked on the quit button.
	if event == but
		break
	end
	# The user clicked on the 'amplitude' slider. Now we need
	# to go to gtk-server again to ask for the new slider's value.
	if event == sliA
		# here we get the new value
		amp_t = gtk_send(fifo,"gtk_range_get_value $sliA")
		amp = float(amp_t)
		# plot again with the new sine values
		dotimeplot(t,amp,fre,pha)
		dofreqplot(f,amp,fre)
	end
	# Frequency slider.
	if event == sliF
		fre_t = gtk_send(fifo,"gtk_range_get_value $sliF")
		fre = float(fre_t)
		dotimeplot(t,amp,fre,pha)
		dofreqplot(f,amp,fre)
        dophaseplot(f,fre,pha)
	end
	# Phase slider.
	if event == sliP
		pha_t = gtk_send(fifo,"gtk_range_get_value $sliP")
		pha = float(pha_t)
		dotimeplot(t,amp,fre,pha)
        dophaseplot(f,fre,pha)
	end
end

# Clean up -- close all figures and tell gtk-server to quit.
closeall()
gtk_exit(fifo)
